// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: transportinstruction.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_transportinstruction_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_transportinstruction_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3021000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3021012 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/message_lite.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_transportinstruction_2eproto
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct TableStruct_transportinstruction_2eproto {
  static const uint32_t offsets[];
};
namespace TransportBuffers {
class Instruction;
struct InstructionDefaultTypeInternal;
extern InstructionDefaultTypeInternal _Instruction_default_instance_;
}  // namespace TransportBuffers
PROTOBUF_NAMESPACE_OPEN
template<> ::TransportBuffers::Instruction* Arena::CreateMaybeMessage<::TransportBuffers::Instruction>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace TransportBuffers {

// ===================================================================

class Instruction final :
    public ::PROTOBUF_NAMESPACE_ID::MessageLite /* @@protoc_insertion_point(class_definition:TransportBuffers.Instruction) */ {
 public:
  inline Instruction() : Instruction(nullptr) {}
  ~Instruction() override;
  explicit PROTOBUF_CONSTEXPR Instruction(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  Instruction(const Instruction& from);
  Instruction(Instruction&& from) noexcept
    : Instruction() {
    *this = ::std::move(from);
  }

  inline Instruction& operator=(const Instruction& from) {
    CopyFrom(from);
    return *this;
  }
  inline Instruction& operator=(Instruction&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const std::string& unknown_fields() const {
    return _internal_metadata_.unknown_fields<std::string>(::PROTOBUF_NAMESPACE_ID::internal::GetEmptyString);
  }
  inline std::string* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<std::string>();
  }

  static const Instruction& default_instance() {
    return *internal_default_instance();
  }
  static inline const Instruction* internal_default_instance() {
    return reinterpret_cast<const Instruction*>(
               &_Instruction_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(Instruction& a, Instruction& b) {
    a.Swap(&b);
  }
  inline void Swap(Instruction* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Instruction* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  Instruction* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<Instruction>(arena);
  }
  void CheckTypeAndMergeFrom(const ::PROTOBUF_NAMESPACE_ID::MessageLite& from)  final;
  void CopyFrom(const Instruction& from);
  void MergeFrom(const Instruction& from);
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const;
  void InternalSwap(Instruction* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "TransportBuffers.Instruction";
  }
  protected:
  explicit Instruction(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  std::string GetTypeName() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kDiffFieldNumber = 6,
    kChaffFieldNumber = 7,
    kOldNumFieldNumber = 2,
    kNewNumFieldNumber = 3,
    kAckNumFieldNumber = 4,
    kThrowawayNumFieldNumber = 5,
    kProtocolVersionFieldNumber = 1,
  };
  // optional bytes diff = 6;
  bool has_diff() const;
  private:
  bool _internal_has_diff() const;
  public:
  void clear_diff();
  const std::string& diff() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_diff(ArgT0&& arg0, ArgT... args);
  std::string* mutable_diff();
  PROTOBUF_NODISCARD std::string* release_diff();
  void set_allocated_diff(std::string* diff);
  private:
  const std::string& _internal_diff() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_diff(const std::string& value);
  std::string* _internal_mutable_diff();
  public:

  // optional bytes chaff = 7;
  bool has_chaff() const;
  private:
  bool _internal_has_chaff() const;
  public:
  void clear_chaff();
  const std::string& chaff() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_chaff(ArgT0&& arg0, ArgT... args);
  std::string* mutable_chaff();
  PROTOBUF_NODISCARD std::string* release_chaff();
  void set_allocated_chaff(std::string* chaff);
  private:
  const std::string& _internal_chaff() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_chaff(const std::string& value);
  std::string* _internal_mutable_chaff();
  public:

  // optional uint64 old_num = 2;
  bool has_old_num() const;
  private:
  bool _internal_has_old_num() const;
  public:
  void clear_old_num();
  uint64_t old_num() const;
  void set_old_num(uint64_t value);
  private:
  uint64_t _internal_old_num() const;
  void _internal_set_old_num(uint64_t value);
  public:

  // optional uint64 new_num = 3;
  bool has_new_num() const;
  private:
  bool _internal_has_new_num() const;
  public:
  void clear_new_num();
  uint64_t new_num() const;
  void set_new_num(uint64_t value);
  private:
  uint64_t _internal_new_num() const;
  void _internal_set_new_num(uint64_t value);
  public:

  // optional uint64 ack_num = 4;
  bool has_ack_num() const;
  private:
  bool _internal_has_ack_num() const;
  public:
  void clear_ack_num();
  uint64_t ack_num() const;
  void set_ack_num(uint64_t value);
  private:
  uint64_t _internal_ack_num() const;
  void _internal_set_ack_num(uint64_t value);
  public:

  // optional uint64 throwaway_num = 5;
  bool has_throwaway_num() const;
  private:
  bool _internal_has_throwaway_num() const;
  public:
  void clear_throwaway_num();
  uint64_t throwaway_num() const;
  void set_throwaway_num(uint64_t value);
  private:
  uint64_t _internal_throwaway_num() const;
  void _internal_set_throwaway_num(uint64_t value);
  public:

  // optional uint32 protocol_version = 1;
  bool has_protocol_version() const;
  private:
  bool _internal_has_protocol_version() const;
  public:
  void clear_protocol_version();
  uint32_t protocol_version() const;
  void set_protocol_version(uint32_t value);
  private:
  uint32_t _internal_protocol_version() const;
  void _internal_set_protocol_version(uint32_t value);
  public:

  // @@protoc_insertion_point(class_scope:TransportBuffers.Instruction)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
    ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr diff_;
    ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr chaff_;
    uint64_t old_num_;
    uint64_t new_num_;
    uint64_t ack_num_;
    uint64_t throwaway_num_;
    uint32_t protocol_version_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_transportinstruction_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// Instruction

// optional uint32 protocol_version = 1;
inline bool Instruction::_internal_has_protocol_version() const {
  bool value = (_impl_._has_bits_[0] & 0x00000040u) != 0;
  return value;
}
inline bool Instruction::has_protocol_version() const {
  return _internal_has_protocol_version();
}
inline void Instruction::clear_protocol_version() {
  _impl_.protocol_version_ = 0u;
  _impl_._has_bits_[0] &= ~0x00000040u;
}
inline uint32_t Instruction::_internal_protocol_version() const {
  return _impl_.protocol_version_;
}
inline uint32_t Instruction::protocol_version() const {
  // @@protoc_insertion_point(field_get:TransportBuffers.Instruction.protocol_version)
  return _internal_protocol_version();
}
inline void Instruction::_internal_set_protocol_version(uint32_t value) {
  _impl_._has_bits_[0] |= 0x00000040u;
  _impl_.protocol_version_ = value;
}
inline void Instruction::set_protocol_version(uint32_t value) {
  _internal_set_protocol_version(value);
  // @@protoc_insertion_point(field_set:TransportBuffers.Instruction.protocol_version)
}

// optional uint64 old_num = 2;
inline bool Instruction::_internal_has_old_num() const {
  bool value = (_impl_._has_bits_[0] & 0x00000004u) != 0;
  return value;
}
inline bool Instruction::has_old_num() const {
  return _internal_has_old_num();
}
inline void Instruction::clear_old_num() {
  _impl_.old_num_ = uint64_t{0u};
  _impl_._has_bits_[0] &= ~0x00000004u;
}
inline uint64_t Instruction::_internal_old_num() const {
  return _impl_.old_num_;
}
inline uint64_t Instruction::old_num() const {
  // @@protoc_insertion_point(field_get:TransportBuffers.Instruction.old_num)
  return _internal_old_num();
}
inline void Instruction::_internal_set_old_num(uint64_t value) {
  _impl_._has_bits_[0] |= 0x00000004u;
  _impl_.old_num_ = value;
}
inline void Instruction::set_old_num(uint64_t value) {
  _internal_set_old_num(value);
  // @@protoc_insertion_point(field_set:TransportBuffers.Instruction.old_num)
}

// optional uint64 new_num = 3;
inline bool Instruction::_internal_has_new_num() const {
  bool value = (_impl_._has_bits_[0] & 0x00000008u) != 0;
  return value;
}
inline bool Instruction::has_new_num() const {
  return _internal_has_new_num();
}
inline void Instruction::clear_new_num() {
  _impl_.new_num_ = uint64_t{0u};
  _impl_._has_bits_[0] &= ~0x00000008u;
}
inline uint64_t Instruction::_internal_new_num() const {
  return _impl_.new_num_;
}
inline uint64_t Instruction::new_num() const {
  // @@protoc_insertion_point(field_get:TransportBuffers.Instruction.new_num)
  return _internal_new_num();
}
inline void Instruction::_internal_set_new_num(uint64_t value) {
  _impl_._has_bits_[0] |= 0x00000008u;
  _impl_.new_num_ = value;
}
inline void Instruction::set_new_num(uint64_t value) {
  _internal_set_new_num(value);
  // @@protoc_insertion_point(field_set:TransportBuffers.Instruction.new_num)
}

// optional uint64 ack_num = 4;
inline bool Instruction::_internal_has_ack_num() const {
  bool value = (_impl_._has_bits_[0] & 0x00000010u) != 0;
  return value;
}
inline bool Instruction::has_ack_num() const {
  return _internal_has_ack_num();
}
inline void Instruction::clear_ack_num() {
  _impl_.ack_num_ = uint64_t{0u};
  _impl_._has_bits_[0] &= ~0x00000010u;
}
inline uint64_t Instruction::_internal_ack_num() const {
  return _impl_.ack_num_;
}
inline uint64_t Instruction::ack_num() const {
  // @@protoc_insertion_point(field_get:TransportBuffers.Instruction.ack_num)
  return _internal_ack_num();
}
inline void Instruction::_internal_set_ack_num(uint64_t value) {
  _impl_._has_bits_[0] |= 0x00000010u;
  _impl_.ack_num_ = value;
}
inline void Instruction::set_ack_num(uint64_t value) {
  _internal_set_ack_num(value);
  // @@protoc_insertion_point(field_set:TransportBuffers.Instruction.ack_num)
}

// optional uint64 throwaway_num = 5;
inline bool Instruction::_internal_has_throwaway_num() const {
  bool value = (_impl_._has_bits_[0] & 0x00000020u) != 0;
  return value;
}
inline bool Instruction::has_throwaway_num() const {
  return _internal_has_throwaway_num();
}
inline void Instruction::clear_throwaway_num() {
  _impl_.throwaway_num_ = uint64_t{0u};
  _impl_._has_bits_[0] &= ~0x00000020u;
}
inline uint64_t Instruction::_internal_throwaway_num() const {
  return _impl_.throwaway_num_;
}
inline uint64_t Instruction::throwaway_num() const {
  // @@protoc_insertion_point(field_get:TransportBuffers.Instruction.throwaway_num)
  return _internal_throwaway_num();
}
inline void Instruction::_internal_set_throwaway_num(uint64_t value) {
  _impl_._has_bits_[0] |= 0x00000020u;
  _impl_.throwaway_num_ = value;
}
inline void Instruction::set_throwaway_num(uint64_t value) {
  _internal_set_throwaway_num(value);
  // @@protoc_insertion_point(field_set:TransportBuffers.Instruction.throwaway_num)
}

// optional bytes diff = 6;
inline bool Instruction::_internal_has_diff() const {
  bool value = (_impl_._has_bits_[0] & 0x00000001u) != 0;
  return value;
}
inline bool Instruction::has_diff() const {
  return _internal_has_diff();
}
inline void Instruction::clear_diff() {
  _impl_.diff_.ClearToEmpty();
  _impl_._has_bits_[0] &= ~0x00000001u;
}
inline const std::string& Instruction::diff() const {
  // @@protoc_insertion_point(field_get:TransportBuffers.Instruction.diff)
  return _internal_diff();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void Instruction::set_diff(ArgT0&& arg0, ArgT... args) {
 _impl_._has_bits_[0] |= 0x00000001u;
 _impl_.diff_.SetBytes(static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:TransportBuffers.Instruction.diff)
}
inline std::string* Instruction::mutable_diff() {
  std::string* _s = _internal_mutable_diff();
  // @@protoc_insertion_point(field_mutable:TransportBuffers.Instruction.diff)
  return _s;
}
inline const std::string& Instruction::_internal_diff() const {
  return _impl_.diff_.Get();
}
inline void Instruction::_internal_set_diff(const std::string& value) {
  _impl_._has_bits_[0] |= 0x00000001u;
  _impl_.diff_.Set(value, GetArenaForAllocation());
}
inline std::string* Instruction::_internal_mutable_diff() {
  _impl_._has_bits_[0] |= 0x00000001u;
  return _impl_.diff_.Mutable(GetArenaForAllocation());
}
inline std::string* Instruction::release_diff() {
  // @@protoc_insertion_point(field_release:TransportBuffers.Instruction.diff)
  if (!_internal_has_diff()) {
    return nullptr;
  }
  _impl_._has_bits_[0] &= ~0x00000001u;
  auto* p = _impl_.diff_.Release();
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (_impl_.diff_.IsDefault()) {
    _impl_.diff_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  return p;
}
inline void Instruction::set_allocated_diff(std::string* diff) {
  if (diff != nullptr) {
    _impl_._has_bits_[0] |= 0x00000001u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001u;
  }
  _impl_.diff_.SetAllocated(diff, GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (_impl_.diff_.IsDefault()) {
    _impl_.diff_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:TransportBuffers.Instruction.diff)
}

// optional bytes chaff = 7;
inline bool Instruction::_internal_has_chaff() const {
  bool value = (_impl_._has_bits_[0] & 0x00000002u) != 0;
  return value;
}
inline bool Instruction::has_chaff() const {
  return _internal_has_chaff();
}
inline void Instruction::clear_chaff() {
  _impl_.chaff_.ClearToEmpty();
  _impl_._has_bits_[0] &= ~0x00000002u;
}
inline const std::string& Instruction::chaff() const {
  // @@protoc_insertion_point(field_get:TransportBuffers.Instruction.chaff)
  return _internal_chaff();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void Instruction::set_chaff(ArgT0&& arg0, ArgT... args) {
 _impl_._has_bits_[0] |= 0x00000002u;
 _impl_.chaff_.SetBytes(static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:TransportBuffers.Instruction.chaff)
}
inline std::string* Instruction::mutable_chaff() {
  std::string* _s = _internal_mutable_chaff();
  // @@protoc_insertion_point(field_mutable:TransportBuffers.Instruction.chaff)
  return _s;
}
inline const std::string& Instruction::_internal_chaff() const {
  return _impl_.chaff_.Get();
}
inline void Instruction::_internal_set_chaff(const std::string& value) {
  _impl_._has_bits_[0] |= 0x00000002u;
  _impl_.chaff_.Set(value, GetArenaForAllocation());
}
inline std::string* Instruction::_internal_mutable_chaff() {
  _impl_._has_bits_[0] |= 0x00000002u;
  return _impl_.chaff_.Mutable(GetArenaForAllocation());
}
inline std::string* Instruction::release_chaff() {
  // @@protoc_insertion_point(field_release:TransportBuffers.Instruction.chaff)
  if (!_internal_has_chaff()) {
    return nullptr;
  }
  _impl_._has_bits_[0] &= ~0x00000002u;
  auto* p = _impl_.chaff_.Release();
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (_impl_.chaff_.IsDefault()) {
    _impl_.chaff_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  return p;
}
inline void Instruction::set_allocated_chaff(std::string* chaff) {
  if (chaff != nullptr) {
    _impl_._has_bits_[0] |= 0x00000002u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000002u;
  }
  _impl_.chaff_.SetAllocated(chaff, GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (_impl_.chaff_.IsDefault()) {
    _impl_.chaff_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:TransportBuffers.Instruction.chaff)
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)

}  // namespace TransportBuffers

// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_transportinstruction_2eproto
